from colin.checks.abstract.labels import LabelCheck


class UsageCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="usage_label_required",
                      message="Label 'usage' has to be specified.",
                      description="A human readable example of container execution.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["usage", "label", "required"],
                      label="usage",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import DeprecatedLabelCheck


class VersionLabelCapitalDeprecatedCheck(DeprecatedLabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="version_label_capital_deprecated",
                      message="Label 'Version' is deprecated.",
                      description="Replace with 'version'.",
                      reference_url="?????",
                      tags=["version", "label", "capital", "deprecated"],
                      old_label="Version",
                      new_label="version")



from colin.checks.abstract.containers import ContainerCheck
from colin.checks.abstract.images import ImageCheck
from colin.checks.result import CheckResult


class NoRootCheck(ContainerCheck, ImageCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="no_root",
                      message="Service should not run as root by default.",
                      description="",
                      reference_url="?????",
                      tags=["root", "user"])

    def check(self, target):
        metadata = target.instance.get_metadata()["Config"]
        root_present = "User" in metadata and metadata["User"] in ["", "0", "root"]

        return CheckResult(ok=not root_present,
                           severity=self.severity,
                           description=self.description,
                           message=self.message,
                           reference_url=self.reference_url,
                           check_name=self.name,
                           logs=[])



from colin.checks.abstract.labels import DeprecatedLabelCheck


class ReleaseLabelCapitalDeprecatedCheck(DeprecatedLabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="release_label_capital_deprecated",
                      message="Label 'Release' is deprecated.",
                      description="Replace with 'release'.",
                      reference_url="?????",
                      tags=["release", "label", "capital", "deprecated"],
                      old_label="Release",
                      new_label="release")



from colin.checks.abstract.labels import DeprecatedLabelCheck


class NameLabelCapitalDeprecatedCheck(DeprecatedLabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="name_label_capital_deprecated",
                      message="Label 'Name' is deprecated.",
                      description="Replace with 'name'.",
                      reference_url="?????",
                      tags=["name", "label", "capital", "deprecated"],
                      old_label="Name",
                      new_label="name")



import re


def check_label(label, required, value_regex, labels):
    """
    Check if the label is required and match the regex

    :param label: str
    :param required: bool (if the presence means pass or not)
    :param value_regex: str
    :param labels: [str]
    :return: bool (required==True: True if the label is present and match the regex if specified)
                    (required==False: True if the label is not present)
    """
    present = labels is not None and label in labels

    if present:
        if required and not value_regex:
            return True
        elif value_regex:
            pattern = re.compile(value_regex)
            return bool(pattern.match(labels[label]))
        else:
            return False

    else:
        return not required



from colin.checks.abstract.labels import LabelCheck


class VcsRefLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="vcs-ref_label",
                      message="Label 'vcs-ref' has to be specified.",
                      description="A 'reference' within the version control repository; e.g. a git commit, or a subversion branch.",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels",
                      tags=["vcs-ref", "vcs", "label", "required"],
                      label="vcs-ref",
                      required=True,
                      value_regex=None)



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import enum
import io
import logging
import os

from conu import DockerBackend, DockerImagePullPolicy
from conu.apidefs.container import Container
from conu.apidefs.image import Image
from docker.errors import NotFound
from dockerfile_parse import DockerfileParser

from ..checks.abstract.containers import ContainerCheck
from ..checks.abstract.dockerfile import DockerfileCheck
from ..checks.abstract.images import ImageCheck
from ..core.exceptions import ColinException

logger = logging.getLogger(__name__)


def is_compatible(target_type, check_instance):
    """
    Check the target compatibility with the check instance.

    :param target_type: TargetType enum, if None, returns True
    :param check_instance: instance of some Check class
    :return: True if the target is None or compatible with the check instance
    """
    if not target_type:
        return True
    return (target_type == TargetType.DOCKERFILE and isinstance(check_instance, DockerfileCheck)) \
           or (target_type == TargetType.CONTAINER and isinstance(check_instance, ContainerCheck)) \
           or (target_type == TargetType.CONTAINER_IMAGE and isinstance(check_instance, ImageCheck))


class Target(object):

    def __init__(self, target, logging_level):
        self.instance = Target._get_target_instance(target, logging_level=logging_level)

    @staticmethod
    def _get_target_instance(target, logging_level):
        """
        Get the Container/Image instance for the given name.
        (Container is the first choice.)
        or DockerfileParser instance if the target is path or file-like object.

        :param target: str
                        or instance of Image/Container
                        or file-like object as Dockerfile
        :return: Target object
        """
        logger.debug("Finding target '{}'.".format(target))

        if isinstance(target, (Image, Container)):
            logger.debug("Target is a conu object.")
            return target
        if isinstance(target, io.IOBase):
            logger.debug("Target is a dockerfile loaded from the file-like object.")
            return DockerfileParser(fileobj=target)
        if os.path.isfile(target):
            logger.debug("Target is a dockerfile.")
            return DockerfileParser(fileobj=open(target))

        with DockerBackend(logging_level=logging_level) as backend:

            try:
                cont = backend.ContainerClass(image=None,
                                              container_id=target)
                logger.debug("Target is a container.")
                return cont
            except NotFound:

                image_name = ImageName.parse(target)
                logger.debug("Finding image '{}' with tag '{}'.".format(image_name.name, image_name.tag))

                if image_name.tag:
                    image = backend.ImageClass(repository=image_name.name,
                                               tag=image_name.tag,
                                               pull_policy=DockerImagePullPolicy.NEVER)
                else:
                    image = backend.ImageClass(repository=image_name.name,
                                               pull_policy=DockerImagePullPolicy.NEVER)

                if image.is_present():
                    logger.debug("Target is an image.")
                    return image
        logger.error("Target is neither image nor container.")
        raise ColinException("Target not found.")

    @property
    def target_type(self):
        """
        Type of the target (image/container/dockerfile)

        :return: TargetType enum
        """
        if isinstance(self.instance, Image):
            return TargetType.CONTAINER_IMAGE
        elif isinstance(self.instance, Container):
            return TargetType.CONTAINER
        elif isinstance(self.instance, DockerfileParser):
            return TargetType.DOCKERFILE
        logger.debug("Target type not found.")
        raise ColinException("Target type not found.")

    @property
    def labels(self):
        """
        Get list of labels from the target instance.

        :return: [str]
        """
        if self.target_type == TargetType.DOCKERFILE:
            return self.instance.labels
        return self.instance.get_metadata()["Config"]["Labels"]


class TargetType(enum.Enum):
    DOCKERFILE = 0
    CONTAINER = 1
    CONTAINER_IMAGE = 2


class ImageName(object):
    def __init__(self, registry=None, namespace=None, repository=None, tag=None, digest=None):
        self.registry = registry
        self.namespace = namespace
        self.repository = repository
        self.tag = tag
        self.digest = digest

    @classmethod
    def parse(cls, image_name):
        """
        Get the instance of ImageName from the string representation.

        :param image_name: str (any possible form of image name)
        :return: ImageName instance
        """
        result = cls()

        # registry.org/namespace/repo:tag
        s = image_name.split('/', 2)

        if len(s) == 2:
            if '.' in s[0] or ':' in s[0]:
                result.registry = s[0]
            else:
                result.namespace = s[0]
        elif len(s) == 3:
            result.registry = s[0]
            result.namespace = s[1]
        result.repository = s[-1]

        try:
            result.repository, result.digest = result.repository.rsplit("@", 1)
        except ValueError:
            pass

        try:
            result.repository, result.tag = result.repository.rsplit(":", 1)
        except ValueError:
            pass

        return result

    def __str__(self):
        return "Image: registry='{}' namespace='{}' " \
               "repository='{}' tag='{}' digest='{}'".format(self.registry,
                                                             self.namespace,
                                                             self.repository,
                                                             self.tag,
                                                             self.digest)

    @property
    def name(self):
        """
        Get the string representation of the image
        (registry, namespace, repository and digest together).

        :return: str
        """
        name_parts = []
        if self.registry:
            name_parts.append(self.registry)

        if self.namespace:
            name_parts.append(self.namespace)

        if self.repository:
            name_parts.append(self.repository)
        name = "/".join(name_parts)

        if self.digest:
            name += "@{}".format(self.digest)

        return name



from colin.checks.abstract.labels import DeprecatedLabelCheck


class BZComponentDeprecatedCheck(DeprecatedLabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="bzcomponent_deprecated",
                      message="Label 'BZComponent' is deprecated.",
                      description="Replace with 'com.redhat.component'.",
                      reference_url="?????",
                      tags=["com.redhat.component", "bzcomponent", "label", "deprecated"],
                      old_label="BZComponent",
                      new_label="com.redhat.component")



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import json

import six
from six import iteritems

from ..core.constant import (COLOURS, ERROR, FAILED, OPTIONAL, OUTPUT_CHARS,
                             PASSED, REQUIRED, WARNING)


class CheckResult(object):

    def __init__(self, ok, description, message, reference_url, check_name, severity, logs):
        self.ok = ok
        self.description = description
        self.message = message
        self.reference_url = reference_url
        self.check_name = check_name
        self.severity = severity
        self.logs = logs

    @property
    def status(self):
        statuses = {REQUIRED: (PASSED, FAILED),
                    OPTIONAL: (PASSED, WARNING)}
        status_ok, status_nok = statuses[self.severity]

        return status_ok if self.ok else status_nok

    def __str__(self):
        return "{}:{}".format(self.status,
                              self.message)


class DockerfileCheckResult(CheckResult):

    def __init__(self, ok, description, message, reference_url, check_name, severity, lines=None,
                 correction_diff=None):
        super(self.__class__, self) \
            .__init__(ok, description, message, reference_url, check_name, severity)
        self.lines = lines
        self.correction_diff = correction_diff


class CheckResults(object):

    def __init__(self, results):
        self._results = results
        self._generated = False
        self._generated_result = {}

    @property
    def _dict_of_results(self):
        """
        Get the dictionary representation of results

        :return: dict (str -> dict (str -> str))
        """
        result_json = {}
        for group, results in self.results:
            result_list = []
            for r in results:
                result_list.append({
                    'name': r.check_name,
                    'ok': r.ok,
                    'status': r.status,
                    'description': r.description,
                    'message': r.message,
                    'reference_url': r.reference_url,
                    'severity': r.severity,
                    'logs': r.logs,
                })
            result_json[group] = result_list
        return result_json

    @property
    def json(self):
        """
        Get the json representation of results

        :return: str
        """
        return json.dumps(self._dict_of_results, indent=4)

    @property
    def all_results(self):
        """
        Get the list of all results

        :return: list of Result instances
        """
        result = []
        for _, checks in self.results:
            result += checks
        return result

    def save_json_to_file(self, file):
        json.dump(obj=self._dict_of_results,
                  fp=file,
                  indent=4)

    @property
    def results(self):
        """
        Get the result generator

        :return: Generator of group generator of results
        """
        if self._generated:
            return iteritems(self._generated_result)
        else:
            return self._group_generator()

    @property
    def statistics(self):
        """
        Get the dictionary with the count of the check-statuses

        :return: dict(str -> int)
        """
        result = {}
        for r in self.all_results:
            result.setdefault(r.status, 0)
            result[r.status] += 1
        return result

    @property
    def ok(self):
        """
        If the results ended without any error


        :return: True, if there is no check which ends with error status
        """
        return ERROR not in self.statistics

    @property
    def fail(self):
        """
        If the results ended without any fail


        :return: True, if there is no check which ends with fail status
        """
        return FAILED in self.statistics

    def _group_generator(self):
        """
        Forward original generator of result groups, but saves the value for the future use.

        :return: yields the group generator
        """
        for group, group_result in self._results:
            self._generated_result.setdefault(group, [])
            yield group, self._result_generator(group=group,
                                                results=group_result)
        self._generated = True

    def _result_generator(self, group, results):
        """
        Forward the result from one generator and saves the value for the future use.

        :param group: str
        :param results: Result object generator
        :return: yields the values from original generator
        """
        for r in results:
            self._generated_result[group].append(r)
            yield r

    def generate_pretty_output(self, stat, verbose, output_function):
        """
        Send the formated to the provided function

        :param stat: if True print stat instead of full output
        :param verbose: bool
        :param output_function: function to send output to
        """
        for group, check_results in self.results:

            group_title_printed = False
            for r in check_results:

                if not group_title_printed:
                    output_function("{}:".format(group.upper()),
                                    nl=not stat)
                    group_title_printed = True

                if stat:
                    output_function(OUTPUT_CHARS[r.status],
                                    fg=COLOURS[r.status],
                                    nl=False)
                else:
                    output_function(str(r), fg=COLOURS[r.status])
                    if verbose:
                        output_function("  -> {}\n"
                                        "  -> {}".format(r.description,
                                                         r.reference_url),
                                        fg=COLOURS[r.status])

            if group_title_printed and stat:
                output_function("")

        if not stat or verbose:
            output_function("")
            for status, count in six.iteritems(self.statistics):
                output_function("{}:{} ".format(status, count), nl=False)
            output_function("")

    def get_pretty_string(self, stat, verbose):
        """
        Pretty string representation of the results

        :param stat: bool
        :param verbose: bool
        :return: str
        """
        pretty_output = _PrettyOutputToStr()
        self.generate_pretty_output(stat=stat,
                                    verbose=verbose,
                                    output_function=pretty_output.save_output)
        return pretty_output.result


class FailedCheckResult(CheckResult):

    def __init__(self, check, exception):
        super(self.__class__, self) \
            .__init__(ok=False,
                      message=check.message,
                      description=str(exception),
                      reference_url="",
                      check_name=check.name,
                      severity=check.severity,
                      logs=[str(exception)]
                      )

    @property
    def status(self):
        return ERROR


class _PrettyOutputToStr(object):

    def __init__(self):
        self.result = ""

    def save_output(self, text=None, fg=None, nl=True):
        text = text or ""
        self.result += text
        if nl:
            self.result += "\n"



from colin.checks.abstract.labels import LabelCheck


class VersionCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="version_label_required",
                      message="Label 'version' has to be specified.",
                      description="Version of the image.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["version", "label", "required"],
                      label="version",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class ComRedhatComponentCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="com_redhat_component_label_required",
                      message="Label 'com.redhat.component' has to be specified.",
                      description="The Bugzilla component name where bugs against this container should be reported by users.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["com.redhat.component", "label", "required"],
                      label="com.redhat.component",
                      required=True,
                      value_regex=None)
        # TODO: Check the format



from colin.checks.abstract.labels import DeprecatedLabelCheck


class UninstallLabelCapitalDeprecatedCheck(DeprecatedLabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="uninstall_label_capital_deprecated",
                      message="Label 'UNINSTALL' is deprecated.",
                      description="Replace with 'uninstall'.",
                      reference_url="?????",
                      tags=["uninstall", "label", "capital", "deprecated"],
                      old_label="UNINSTALL",
                      new_label="uninstall")



import click
from click.testing import CliRunner
from colin.cli.colin import check, list_checks, list_rulesets

def _call_colin(fnc, parameters=None):
    runner = CliRunner()
    if not parameters:
        return runner.invoke(fnc)
    else:
        return runner.invoke(fnc, parameters)


def _common_help_options(result):
    assert "-r, --ruleset TEXT" in result.output
    assert "-f, --ruleset-file FILENAME" in result.output
    assert "--debug" in result.output
    assert "--json FILENAME" in result.output
    assert "-t, --tag TEXT" in result.output
    assert "-v, --verbose" in result.output
    assert "-h, --help" in result.output

def test_check_command():
    result = _call_colin(check)
    expected_output = """Usage: check [OPTIONS] TARGET

Error: Missing argument "target".
"""
    assert result.exit_code == 2
    assert result.output == expected_output


def test_check_help_command():
    result = _call_colin(check, parameters=["-h"])
    assert result.exit_code == 0
    _common_help_options(result)
    assert "-s, --stat" in result.output


def test_list_checks():
    result = _call_colin(list_checks)
    expected_output = """LABELS:
maintainer_label_required
   -> Label 'maintainer' has to be specified.
   -> The name and email of the maintainer (usually the submitter).
   -> https://fedoraproject.org/wiki/Container:Guidelines#LABELS
   -> ['maintainer', 'label', 'required']
   -> required

DOCKERFILE:
maintainer_deprecated
   -> 
   -> 
   -> https://docs.docker.com/engine/reference/builder/#maintainer-deprecated
   -> ['maintainer', 'dockerfile', 'deprecated']
   -> required

from_tag_not_latest
   -> In FROM, tag has to be specified and not 'latest'.
   -> Using the 'latest' tag may cause unpredictable builds.It is recommended that a specific tag is used in the FROM.
   -> https://fedoraproject.org/wiki/Container:Guidelines#FROM
   -> ['from', 'dockerfile', 'baseimage', 'latest']
   -> optional

"""
    assert result.exit_code == 0
    assert result.output == expected_output


def test_list_checks_help_command():
    result = _call_colin(list_checks, parameters=["-h"])
    _common_help_options(result)
    assert result.exit_code == 0


def test_list_checks_fedora():
    result = _call_colin(list_checks, parameters=["-r", "default"])
    assert result.exit_code == 0
    assert "maintainer_label_required" in result.output
    assert "maintainer_deprecated" in result.output
    assert "from_tag_not_latest" in result.output


def test_list_rulesets():
    result = _call_colin(list_rulesets)
    expected_output = """fedora
default
"""
    assert result.exit_code == 0
    assert result.output == expected_output


def test_list_rulesets_help_command():
    result = _call_colin(list_rulesets, parameters=["-h"])
    expected_result = """Usage: list-rulesets [OPTIONS]

  List available rulesets.

Options:
  --debug     Enable debugging mode (debugging logs, full tracebacks).
  -h, --help  Show this message and exit.
"""
    assert result.exit_code == 0
    assert result.output == expected_result



from colin.checks.abstract.labels import LabelCheck


class BuildDateLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="build-date_label",
                      message="Label 'build-date' has to be specified.",
                      description="Date/Time image was built as RFC 3339 date-time.",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels",
                      tags=["build-date", "label", "required"],
                      label="build-date",
                      required=True,
                      value_regex=None)
        # TODO: Check the RFC 3339 date-time format



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import json
import tempfile
import colin
import random

def get_results_from_colin_labels_image(ruleset_name=None, ruleset_file=None):
    return colin.run("colin-labels", ruleset_name=ruleset_name, ruleset_file=ruleset_file)


def test_colin_labels_image():
    assert get_results_from_colin_labels_image()


def generate_json_file(colin_json):
    with tempfile.NamedTemporaryFile(dir='/tmp', delete=True) as tmpfile:
        temp_file_name = tmpfile.name

    with open(temp_file_name, 'w') as outfile:
        json.dump(colin_json, outfile, ensure_ascii=True)
    return temp_file_name


def test_specific_ruleset():
    colin_json={"labels": {
        "required": [
            "maintainer",
            "name",
            "com_redhat_component"
        ],
        "optional": [
            "help"
        ]
      },
    }


    with open(generate_json_file(colin_json), "r") as f:
        result = get_results_from_colin_labels_image(ruleset_file=f)
    assert result
    expected_dict = {"maintainer_label_required": "PASS",
                     "name_label_required": "PASS",
                     "com_redhat_component_label_required": "PASS",
                     "help_label": "WARN",
    }
    labels_dict = {}
    for res in result.all_results:
        labels_dict[res.check_name] = res.status
    for key in expected_dict.keys():
        assert labels_dict[key] == expected_dict[key]



from colin.checks.abstract.cmd import CmdCheck


class DockerEnvCheck(CmdCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="dockerenv",
                      message="a",
                      description="b",
                      reference_url="https://docs.docker.com/engine/reference/builder/#maintainer-deprecated",
                      tags=["dockerenv", "cmd", "output"],
                      cmd=['ls', '/.dockerenv'])



from colin.checks.abstract.labels import LabelCheck


class DescriptionLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="io.k8s.description_label",
                      message="Label 'io.k8s.description' has to be specified.",
                      description="Description of the container displayed in Kubernetes",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels/blob/master/vendor/redhat/labels.md",
                      tags=["io.k8s.description", "description", "label", "required"],
                      label="io.k8s.description",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class AuthoritativeSourceUrlLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="authoritative-source-url_label",
                      message="Label 'authoritative-source-url' has to be specified.",
                      description="The authoritative registry in which the image is published.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["authoritative-source-url", "label", "required"],
                      label="authoritative-source-url",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import DeprecatedLabelCheck


class ArchitectureLabelCapitalDeprecatedCheck(DeprecatedLabelCheck):

    def __init__(self):
        super(self.__class__, self). \
            __init__(name="architecture_label_capital_deprecated",
                     message="Label 'Architecture' is deprecated.",
                     description="Replace with 'architecture'.",
                     reference_url="?????",
                     tags=["architecture", "label", "capital", "deprecated"],
                     old_label="Architecture",
                     new_label="architecture")



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import sys

import click as click
import six
from six import iteritems

from ..checks.abstract.abstract_check import AbstractCheck
from ..core.colin import get_checks, run
from ..core.exceptions import ColinException
from ..core.ruleset.ruleset import get_rulesets
from ..version import __version__
from .default_group import DefaultGroup

logger = logging.getLogger("colin.cli")

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(cls=DefaultGroup, context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__, '--version', '-V')
def cli():
    """
    COLIN -- Container Linter
    """
    pass


@click.command(name="check",
               context_settings=CONTEXT_SETTINGS)
@click.argument('target', type=click.STRING)
@click.option('--ruleset', '-r', type=click.STRING, envvar='COLIN_RULESET',
              help="Select a predefined ruleset (e.g. fedora).")
@click.option('--ruleset-file', '-f', type=click.File(mode='r'),
              help="Path to a file to use for validation (by default they are placed in /usr/share/colin/rulesets).")
@click.option('--debug', default=False, is_flag=True,
              help="Enable debugging mode (debugging logs, full tracebacks).")
@click.option('--json', type=click.File(mode='w'),
              help="File to save the output as json to.")
@click.option('--stat', '-s', is_flag=True,
              help="Print statistics instead of full results.")
@click.option('--tag', '-t', multiple=True, type=click.STRING,
              help="Filter checks with the tag.")
@click.option('--verbose', '-v', is_flag=True,
              help="Verbose mode.")
def check(target, ruleset, ruleset_file, debug, json, stat, tag, verbose):
    """
    Check the image/container/dockerfile (default).
    """
    if ruleset and ruleset_file:
        raise click.BadOptionUsage("Options '--ruleset' and '--file-ruleset' cannot be used together.")

    try:
        if not debug:
            logging.basicConfig(stream=six.StringIO())

        log_level = _get_log_level(debug=debug,
                                   verbose=verbose)
        results = run(target=target,
                      ruleset_name=ruleset,
                      ruleset_file=ruleset_file,
                      logging_level=log_level,
                      tags=tag)
        _print_results(results=results, stat=stat, verbose=verbose)

        if json:
            results.save_json_to_file(file=json)

        if not results.ok:
            sys.exit(1)
        elif results.fail:
            sys.exit(3)

    except ColinException as ex:
        logger.error("An error occurred: %r", ex)
        if debug:
            raise
        else:
            raise click.ClickException(str(ex))
    except Exception as ex:
        logger.error("An error occurred: %r", ex)
        if debug:
            raise
        else:
            raise click.ClickException(str(ex))


@click.command(name="list-checks",
               context_settings=CONTEXT_SETTINGS)
@click.option('--ruleset', '-r', type=click.STRING, envvar='COLIN_RULESET',
              help="Select a predefined ruleset (e.g. fedora).")
@click.option('--ruleset-file', '-f', type=click.File(mode='r'),
              help="Path to a file to use for validation (by default they are placed in /usr/share/colin/rulesets).")
@click.option('--debug', default=False, is_flag=True,
              help="Enable debugging mode (debugging logs, full tracebacks).")
@click.option('--json', type=click.File(mode='w'),
              help="File to save the output as json to.")
@click.option('--tag', '-t', multiple=True, type=click.STRING,
              help="Filter checks with the tag.")
@click.option('--verbose', '-v', is_flag=True,
              help="Verbose mode.")
def list_checks(ruleset, ruleset_file, debug, json, tag, verbose):
    """
    Print the checks.
    """
    if ruleset and ruleset_file:
        raise click.BadOptionUsage("Options '--ruleset' and '--file-ruleset' cannot be used together.")

    try:
        if not debug:
            logging.basicConfig(stream=six.StringIO())

        log_level = _get_log_level(debug=debug,
                                   verbose=verbose)

        checks = get_checks(ruleset_name=ruleset,
                            ruleset_file=ruleset_file,
                            logging_level=log_level,
                            tags=tag)
        _print_checks(checks=checks)

        if json:
            AbstractCheck.save_checks_to_json(file=json, checks=checks)
    except ColinException as ex:
        logger.error("An error occurred: %r", ex)
        if debug:
            raise
        else:
            raise click.ClickException(str(ex))
    except Exception as ex:
        logger.error("An error occurred: %r", ex)
        if debug:
            raise
        else:
            raise click.ClickException(str(ex))


@click.command(name="list-rulesets",
               context_settings=CONTEXT_SETTINGS)
@click.option('--debug', default=False, is_flag=True,
              help="Enable debugging mode (debugging logs, full tracebacks).")
def list_rulesets(debug):
    """
    List available rulesets.
    """
    try:
        rulesets = get_rulesets()
        for r in rulesets:
            click.echo(r)
    except Exception as ex:
        logger.error("An error occurred: %r", ex)
        if debug:
            raise
        else:
            raise click.ClickException(str(ex))


cli.add_command(check)
cli.add_command(list_checks)
cli.add_command(list_rulesets)
cli.set_default_command(check)


def _print_results(results, stat=False, verbose=False):
    """
    Prints the results to the stdout

    :type verbose: bool
    :param results: generator of group results
    :param stat: if True print stat instead of full output
    """
    results.generate_pretty_output(stat=stat,
                                   verbose=verbose,
                                   output_function=click.secho)


def _print_checks(checks):
    for (group, group_checks) in iteritems(checks):

        group_title_printed = False
        for check in group_checks:

            if not group_title_printed:
                click.secho("{}:".format(group.upper()))
                group_title_printed = True

            click.echo(str(check))


def _get_log_level(debug, verbose):
    return logging.DEBUG if debug else logging.NOTSET


if __name__ == '__main__':
    cli()



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import pytest
from colin.core.target import ImageName


@pytest.mark.parametrize("string_input,image_result", [
    ("fedora", (None, None, 'fedora', None, None)),
    ("fedora:27", (None, None, 'fedora', '27', None)),
    ("docker.io/fedora", ('docker.io', None, 'fedora', None, None)),
    ("docker.io/fedora:latest", ('docker.io', None, 'fedora', 'latest', None)),
    ("docker.io/modularitycontainers/conu", ('docker.io', 'modularitycontainers', 'conu', None, None)),
    ("docker.io/centos/postgresql-96-centos7",
     ('docker.io', 'centos', 'postgresql-96-centos7', None, None)),
    ("some-registry.example.com:8888/image6",
     ('some-registry.example.com:8888', None, 'image6', None, None)),
    ("some-registry.example.com:8888/image6:some-example-6.10-something-26365-20180322014912",
     ('some-registry.example.com:8888', None, 'image6', 'some-example-6.10-something-26365-20180322014912', None)),
    ("fedora@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
     (None, None, 'fedora', None, 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')),
    ("docker.io/fedora@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
     ('docker.io', None, 'fedora', None, 'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')),
    ("docker.io/centos/postgresql-96-centos7@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
     ('docker.io', 'centos', 'postgresql-96-centos7', None,
      'sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')),
])
def test_image_class(string_input, image_result):
    image_name = ImageName.parse(string_input)
    registry, namespace, repository, tag, digest = image_result
    assert image_name.registry == registry
    assert image_name.namespace == namespace
    assert image_name.repository == repository
    assert image_name.tag == tag
    assert image_name.digest == digest


@pytest.mark.parametrize("string_input, name_result, tag_result", [
    ("fedora", "fedora", None),
    ("fedora:27", "fedora", "27"),
    ("docker.io/fedora", "docker.io/fedora", None),
    ("docker.io/fedora:latest", "docker.io/fedora", "latest"),
    ("docker.io/modularitycontainers/conu", "docker.io/modularitycontainers/conu", None),
    ("docker.io/centos/postgresql-96-centos7",
     "docker.io/centos/postgresql-96-centos7", None),
    ("some-registry.example.com:8888/image6",
     "some-registry.example.com:8888/image6", None),
    ("some-registry.example.com:8888/image6:some-example-6.10-something-26365-20180322014912",
     "some-registry.example.com:8888/image6", 'some-example-6.10-something-26365-20180322014912'),
    ("fedora@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
     "fedora@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855", None),
    ("docker.io/fedora@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
     "docker.io/fedora@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855", None),
    ("docker.io/centos/postgresql-96-centos7@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
     "docker.io/centos/postgresql-96-centos7@sha256:e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855", None),
])
def test_image_class_name_tag(string_input, name_result, tag_result):
    image_name = ImageName.parse(string_input)
    assert image_name.name == name_result
    assert image_name.tag == tag_result



from colin.checks.abstract.dockerfile import InstructionCountCheck


class MaintainerDeprecatedCheck(InstructionCountCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="maintainer_deprecated",
                      message="",
                      description="",
                      reference_url="https://docs.docker.com/engine/reference/builder/#maintainer-deprecated",
                      tags=["maintainer", "dockerfile", "deprecated"],
                      instruction="MAINTAINER",
                      max_count=0)



from colin.checks.abstract.labels import LabelCheck


class DescriptionScopeLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="distribution-scope_label",
                      message="Label 'distribution-scope' has to be specified.",
                      description="Scope of intended distribution of the image. (private/authoritative-source-only/restricted/public)",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels",
                      tags=["distribution-scope", "label", "required"],
                      label="distribution-scope",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class IoOpenShiftTagsCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="io_k8s_display-name_label_required",
                      message="Label 'io.k8s.display-name' has to be specified.",
                      description="This label is used to display a human readable name of an image inside the Image / Repo Overview page.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["io.k8s.display-name", "label", "required"],
                      label="io.k8s.display-name",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import DeprecatedLabelCheck


class InstallLabelCapitalDeprecatedCheck(DeprecatedLabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="install_label_capital_deprecated",
                      message="Label 'INSTALL' is deprecated.",
                      description="Replace with 'install'.",
                      reference_url="?????",
                      tags=["install", "label", "capital", "deprecated"],
                      old_label="INSTALL",
                      new_label="install")



from colin.checks.abstract.filesystem import FileSystemCheck


class HelpFileOrReadmeCheck(FileSystemCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="help_file_or_readme_required",
                      message="The 'helpfile' has to be provided.",
                      description="Just like traditional packages, containers need "
                                  "some 'man page' information about how they are to be used,"
                                  " configured, and integrated into a larger stack.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#Help_File",
                      files=['/help.1', '/README.md'],
                      tags=['filesystem', 'helpfile', 'man'],
                      all_must_be_present=False)



from colin.checks.abstract.images import ImageCheck


class RunUsageCheck(ImageCheck):
    def __init__(self):
        super(__class__, self) \
            .__init__(name="run_usage",
                      message="Image can be run with the command specified in the usage label.",
                      description="The label usage is used for the invoking the image.",
                      reference_url="",
                      tags=["usage", "cmd", "image"])

    def check(self, target):
        if "usage" not in target.labels:
            return None





# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import colin

def get_results_from_colin_labels_image():
    return colin.run("colin-labels", ruleset_name="fedora")

def test_colin_image():
    assert get_results_from_colin_labels_image()

def test_labels_in_image():
    result = get_results_from_colin_labels_image()
    assert result
    expected_dict = {"maintainer_label_required": "PASS",
                     "name_label_required": "PASS",
                     "com_redhat_component_label_required": "PASS",
                     "summary_label_required": "PASS",
                     "version_label_required": "PASS",
                     "usage_label_required": "FAIL",
                     "release_label": "FAIL",
                     "architecture_label": "FAIL",
                     "url_label": "WARN",
                     "help_label": "WARN",
                     "build-date_label": "WARN",
                     "distribution-scope_label": "WARN",
                     "vcs-ref_label": "WARN",
                     "vcs-type_label": "WARN",
                     "description_label": "PASS",
                     "io.k8s.description_label": "PASS",
                     "vcs-url_label": "WARN",
                     "io.openshift.expose-services_label": "WARN",
                     "help_file_or_readme_required": "ERROR",
                     "cmd_or_entrypoint": "PASS",
                     "no_root": "FAIL",
    }
    labels_dict = {}
    for res in result.all_results:
        labels_dict[res.check_name] = res.status
    for key in expected_dict.keys():
        assert labels_dict[key] == expected_dict[key]



from colin.checks.abstract.labels import LabelCheck


class VcsTypeLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="vcs-type_label",
                      message="Label 'vcs-type' has to be specified.",
                      description="The type of version control used by the container source."
                                  "Generally one of git, hg, svn, bzr, cvs",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels",
                      tags=["vcs-type", "vcs", "label", "required"],
                      label="vcs-type",
                      required=True,
                      value_regex=None)



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from six import iteritems

from ..checks.result import CheckResults, FailedCheckResult

logger = logging.getLogger(__name__)


def go_through_checks(target, checks):
    logger.debug("Going through checks.")
    results = _group_generator(target=target,
                               checks=checks)
    return CheckResults(results=results)


def _result_generator(target, checks):
    for check in checks:
        logger.debug("Checking {}".format(check.name))
        try:
            yield check.check(target)
        except Exception as ex:
            logger.warning("There occurred an error when executing the check. ({})".format(ex))
            yield FailedCheckResult(check, ex)


def _group_generator(target, checks):
    for (group, group_checks) in iteritems(checks):
        logger.debug("Checking group: {}".format(group))
        yield group, _result_generator(target=target,
                                       checks=group_checks)



from colin.checks.abstract.labels import LabelCheck


class BuildHostLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="com.redhat.build-host_label",
                      message="Label 'com.redhat.build-host' has to be specified.",
                      description="The build host used to create an image for internal use and auditability, similar to the use in RPM.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["com.redhat.build-host", "build-host", "label", "required"],
                      label="com.redhat.build-host",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class HelpLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="help_label",
                      message="Label 'help' has to be specified.",
                      description="A runnable command which results in display of Help information.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["help", "label", "required", "optional"],
                      label="help",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class ArchitectureLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="architecture_label",
                      message="Label 'architecture' has to be specified.",
                      description="Architecture the software in the image should target. "
                                  "(Optional: if omitted, it will be built "
                                  "for all supported Fedora Architectures)",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["architecture", "label", "required", "optional"],
                      label="architecture",
                      required=True,
                      value_regex=None)



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ...core.exceptions import ColinException
from ..result import CheckResult
from .containers import ContainerCheck
from .images import ImageCheck


class FileSystemCheck(ContainerCheck, ImageCheck):

    def __init__(self, name, message, description, reference_url, tags, files, all_must_be_present):
        super(FileSystemCheck, self) \
            .__init__(name, message, description, reference_url, tags)
        self.files = files
        self.all_must_be_present = all_must_be_present

    def check(self, target):
        try:
            with target.instance.mount() as fs:
                passed = self.all_must_be_present

                logs = []
                for f in self.files:
                    try:
                        f_present = fs.file_is_present(f)
                        logs.append("File '{}' is {}present."
                                    .format(f, "" if f_present else "not "))
                    except IOError as ex:
                        f_present = False
                        logs.append("Error: {}".format(str(ex)))

                    if self.all_must_be_present:
                        passed = f_present and passed
                    else:
                        passed = f_present or passed

                return CheckResult(ok=passed,
                                   severity=self.severity,
                                   description=self.description,
                                   message=self.message,
                                   reference_url=self.reference_url,
                                   check_name=self.name,
                                   logs=logs)
        except Exception as ex:
            raise ColinException("Problem with mounting filesystem with atomic. ({})"
                                 .format(str(ex)))



import logging

from colin.checks.abstract.containers import ContainerCheck
from colin.checks.abstract.images import ImageCheck
from colin.checks.result import CheckResult

logger = logging.getLogger(__name__)


class CmdOrEntrypointCheck(ContainerCheck, ImageCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="cmd_or_entrypoint",
                      message="Cmd or Entrypoint has to be specified",
                      description="",
                      reference_url="?????",
                      tags=["cmd", "entrypoint", "required"])

    def check(self, target):
        metadata = target.instance.get_metadata()["Config"]
        cmd_present = "Cmd" in metadata and metadata["Cmd"]
        msg_cmd_present = "Cmd {}specified.".format("" if cmd_present else "not ")
        logger.debug(msg_cmd_present)

        entrypoint_present = "Entrypoint" in metadata and metadata["Entrypoint"]
        msg_entrypoint_present = "Entrypoint {}specified.".format("" if entrypoint_present else "not ")
        logger.debug(msg_entrypoint_present)

        passed = cmd_present or entrypoint_present
        return CheckResult(ok=passed,
                           severity=self.severity,
                           description=self.description,
                           message=self.message,
                           reference_url=self.reference_url,
                           check_name=self.name,
                           logs=[msg_cmd_present, msg_entrypoint_present])



from colin.checks.abstract.labels import LabelCheck


class VcsUrlLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="vcs-url_label",
                      message="Label 'vcs-url' has to be specified.",
                      description="URL of the version control repository.",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels",
                      tags=["vcs-url", "vcs", "label", "optional"],
                      label="vcs-url",
                      required=True,
                      value_regex=None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from setuptools import find_packages, setup

BASE_PATH = os.path.dirname(__file__)

# https://packaging.python.org/guides/single-sourcing-package-version/
version = {}
with open("./colin/version.py") as fp:
    exec(fp.read(), version)

long_description = ''.join(open('README.md').readlines())

setup(
    name='colin',
    version=version["__version__"],
    description="Tool to check generic rules/best-practices for containers/images/dockerfiles.",
    long_description=long_description,
    packages=find_packages(exclude=['examples', 'tests']),
    install_requires=[
        'Click',
        'six',
        'conu>=0.3.0rc0',
        'dockerfile_parse'
    ],
    entry_points='''
        [console_scripts]
        colin=colin.cli.colin:cli
    ''',
    data_files=[("share/colin/rulesets/", ["rulesets/default.json",
                                           "rulesets/fedora.json"]),
                ("share/bash-completion/completions/", ["bash-complete/colin"])],
    license='GPLv3+',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Utilities',
    ],
    keywords='containers,sanity,linter',
    author='Red Hat',
    author_email='flachman@redhat.com',
    url='https://github.com/user-cont/colin',
)



from colin.checks.abstract.dockerfile import DockerfileCheck
from colin.checks.result import CheckResult
from colin.core.target import ImageName


class FromTagCheck(DockerfileCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="from_tag_not_latest",
                      message="In FROM, tag has to be specified and not 'latest'.",
                      description="Using the 'latest' tag may cause unpredictable builds."
                                  "It is recommended that a specific tag is used in the FROM.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#FROM",
                      tags=["from", "dockerfile", "baseimage", "latest"])

    def check(self, target):
        im = ImageName.parse(target.instance.baseimage)
        passed = im.tag and im.tag != "latest"
        return CheckResult(ok=passed,
                           severity=self.severity,
                           description=self.description,
                           message=self.message,
                           reference_url=self.reference_url,
                           check_name=self.name,
                           logs=[])



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import json
import logging
import os

import six
from six import iteritems

from ..constant import JSON, RULESET_DIRECTORY
from ..exceptions import ColinRulesetException
from ..loader import load_check_implementation
from ..target import is_compatible

logger = logging.getLogger(__name__)


class Ruleset(object):

    def __init__(self, ruleset_name=None, ruleset_file=None):
        """
        Load ruleset for colin.

        :param ruleset_name: str (name of the ruleset file (without .json), default is "default"
        :param ruleset_file: fileobj
        """
        if ruleset_file:
            try:
                logger.debug("Loading ruleset from file '{}'.".format(ruleset_file.name))
                self.ruleset_dict = json.load(ruleset_file)
            except Exception as ex:
                msg = "Ruleset file '{}' cannot be loaded.".format(ruleset_file.name)
                logger.error(msg)
                raise ColinRulesetException(msg)
        else:
            try:
                logger.debug("Loading ruleset with the name '{}'.".format(ruleset_name))
                ruleset_path = get_ruleset_file(ruleset=ruleset_name)
                with open(ruleset_path, mode='r') as ruleset_file_obj:
                    self.ruleset_dict = json.load(ruleset_file_obj)
            except ColinRulesetException as ex:
                raise ex
            except Exception as ex:
                file_name = ruleset_path if ruleset_path else ruleset_name
                msg = "Ruleset '{}' cannot be loaded.".format(file_name)

                logger.error(msg)
                raise ColinRulesetException(msg)

    def get_checks(self, target_type, group=None, severity=None, tags=None):
        """
        Get all checks for given type/group/severity/tags.

        :param target_type: TargetType enum
        :param group: str (if not group, get checks from all groups/directories)
        :param severity: str (optional x required)
        :param tags: list of str
        :return: list of check instances
        """
        groups = {}
        for g in self._get_check_groups(group):
            logger.debug("Getting checks for group '{}'.".format(g))
            check_files = []
            for sev, rules in iteritems(self.ruleset_dict[g]):

                if severity and severity != sev:
                    continue

                check_files += Ruleset.get_checks_from_rules(rules=rules,
                                                             group=g,
                                                             target_type=target_type,
                                                             severity=sev,
                                                             tags=tags)
            groups[g] = check_files
        return groups

    @staticmethod
    def get_check_file(group, name):
        """
        Get the check file from given group with given name.

        :param group: str
        :param name: str
        :return: str (path)
        """
        return os.path.join(get_checks_path(), group, name + ".py")

    @staticmethod
    def get_checks_from_rules(rules, group, target_type, severity, tags):
        """
        get check from the list of check items in the resultset file.

        :param rules: [str or dict]
        :param group: str
        :param target_type: TargetType enum
        :param severity: str
        :param tags: [str]
        :return: list of filtered check instances
        """
        rule_items = []
        for rule in rules:

            if isinstance(rule, six.string_types):
                rule_items.append(rule)
            elif isinstance(rule, dict):
                if target_type and target_type.name not in rule["type"]:
                    continue

                rule_items += rule["checks"]

        check_instances = []
        for r in rule_items:
            logger.debug("Loading check instance for {}".format(r))
            check_instances += load_check_implementation(path=Ruleset.get_check_file(group, r))
        result = []
        for check_instance in check_instances:
            check_instance.severity = severity
            if not is_compatible(target_type=target_type, check_instance=check_instance):
                logger.debug(
                    "Check '{}' not compatible with the target type: {}".format(check_instance.name, target_type.name))
                continue

            if tags:
                if not set(tags) < set(check_instance.tags):
                    logger.debug("Check '{}' not passed the tag control: {}".format(check_instance.name,
                                                                                  tags))
                    continue
            result.append(check_instance)
            logger.debug("Check instance {} added.".format(check_instance.name))

        return result

    def _get_check_groups(self, group=None):
        """
        Get check group to validate

        :param group: str (if None, all from the ruleset will be used)
        :return: list of str (group names)
        """
        groups = [g for g in self.ruleset_dict]
        if group:
            if group in groups:
                check_groups = [group]
            else:
                check_groups = []
        else:
            check_groups = groups
        logger.debug("Found groups: {}.".format(check_groups))
        return check_groups


def get_checks_path():
    """
    Get path to checks.

    :return: str (absolute path of directory with checks)
    """
    rel_path = os.path.join(os.pardir, os.pardir, os.pardir, "checks")
    return os.path.abspath(os.path.join(__file__, rel_path))


def get_ruleset_file(ruleset=None):
    """
    Get the ruleset file from name

    :param ruleset: str
    :return: str
    """
    ruleset = ruleset or "default"

    ruleset_directory = get_ruleset_directory()
    ruleset_file = os.path.join(ruleset_directory, ruleset + JSON)

    if os.path.isfile(ruleset_file):
        logger.debug("Ruleset file '{}' found.".format(ruleset_file))
        return ruleset_file

    logger.warning("Ruleset with the name '{}' cannot be found at '{}'."
                   .format(ruleset, ruleset_file))
    raise ColinRulesetException("Ruleset with the name '{}' cannot be found.".format(ruleset))


def get_ruleset_directory():
    """
    Get the directory with ruleset files
    First directory to check:  $HOME/.local/share/colin/rulesets
    Second directory to check: /usr/local/share/colin/rulesets
    :return: str
    """

    local_share = os.path.join(os.path.expanduser("~"),
                               ".local",
                               RULESET_DIRECTORY)
    if os.path.isdir(local_share):
        logger.debug("Local ruleset directory found ('{}').".format(local_share))
        return local_share

    usr_local_share = os.path.join("/usr/local", RULESET_DIRECTORY)
    if os.path.isdir(usr_local_share):
        logger.debug("Global ruleset directory found ('{}').".format(usr_local_share))
        return usr_local_share

    msg = "Ruleset directory cannot be found."
    logger.warning(msg)
    raise ColinRulesetException(msg)


def get_rulesets():
    """"
    Get available rulesets.
    """
    rulesets_dir = get_ruleset_directory()
    ruleset_files = [f[:-len(JSON)] for f in os.listdir(rulesets_dir) if
                     os.path.isfile(os.path.join(rulesets_dir, f)) and f.lower().endswith(JSON)]
    return ruleset_files



from colin.checks.abstract.labels import LabelCheck


class IoOpenshiftExposeServicesLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="io.openshift.expose-services_label",
                      message="Label 'io.openshift.expose-services' has to be specified.",
                      description="port:service pairs separated with comma, e.g. \"8080:http,8443:https\"",
                      reference_url="?????",
                      tags=["io.openshift.expose-services", "label", "optional"],
                      label="io.openshift.expose-services",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class MaintainerCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="maintainer_label_required",
                      message="Label 'maintainer' has to be specified.",
                      description="The name and email of the maintainer (usually the submitter).",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["maintainer", "label", "required"],
                      label="maintainer",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class IoOpenShiftTagsCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="io_openshift_tags_label_required",
                      message="Label 'io.openshift.tags' has to be specified.",
                      description="The primary purpose of this label is to include all relevant search terms for this image.",
                      reference_url="https://fedoraproject.org/wiki/Container:Guidelines#LABELS",
                      tags=["io.openshift.tags", "label", "required"],
                      label="io.openshift.tags",
                      required=True,
                      value_regex=None)



from colin.checks.abstract.labels import LabelCheck


class VendorLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="vendor_label",
                      message="Label 'vendor' has to be specified.",
                      description="'Red Hat, Inc.'",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels/blob/master/vendor/redhat/labels.md",
                      tags=["vendor", "label", "required"],
                      label="vendor",
                      required=True,
                      value_regex="^Red Hat, Inc.$")



from colin.checks.abstract.labels import LabelCheck


class DescriptionLabelCheck(LabelCheck):

    def __init__(self):
        super(self.__class__, self) \
            .__init__(name="description_label",
                      message="Label 'description' has to be specified.",
                      description="Detailed description of the image.",
                      reference_url="https://github.com/projectatomic/ContainerApplicationGenericLabels",
                      tags=["description", "label", "required"],
                      label="description",
                      required=True,
                      value_regex=None)



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

RULESET_DIRECTORY = "share/colin/rulesets/"
JSON = ".json"
MODULE_NAME_IMPORTED_CHECKS = "colin.checks.imported"

PASSED = "PASS"
FAILED = "FAIL"
WARNING = "WARN"
ERROR = "ERROR"

REQUIRED = "required"
OPTIONAL = "optional"

COLOURS = {
    PASSED: "green",
    WARNING: "yellow",
    FAILED: "red",
    ERROR: "red"
}

OUTPUT_CHARS = {
    PASSED: ".",
    WARNING: "!",
    FAILED: "x",
    ERROR: "#"
}



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import inspect
import logging
import os
import warnings

import six

from .constant import MODULE_NAME_IMPORTED_CHECKS

logger = logging.getLogger(__name__)


def _load_module(path):
    module_name = "{}.{}.{}".format(MODULE_NAME_IMPORTED_CHECKS,
                                    os.path.basename(os.path.dirname(path)),
                                    os.path.basename(path))
    if six.PY3:

        from importlib.util import module_from_spec
        from importlib.util import spec_from_file_location

        s = spec_from_file_location(module_name, path)
        m = module_from_spec(s)
        s.loader.exec_module(m)
        return m

    elif six.PY2:
        import imp

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            m = imp.load_source(module_name, path)
        return m


def load_check_implementation(path):
    logger.debug("Getting check(s) from the file '{}'.".format(path))
    m = _load_module(path)

    check_classes = []
    for name, obj in inspect.getmembers(m, inspect.isclass):
        if obj.__module__.startswith(MODULE_NAME_IMPORTED_CHECKS):
            new_check = obj()
            check_classes.append(new_check)
            logger.debug("Check instance '{}' found.".format(new_check.name))
    return check_classes



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from .check_runner import go_through_checks
from .ruleset.ruleset import Ruleset
from .target import Target

logger = logging.getLogger(__name__)


def run(target, group=None, severity=None, tags=None, ruleset_name=None, ruleset_file=None,
        logging_level=logging.WARNING):
    """
    Runs the sanity checks for the target.

    :param target: str
                    or Image/Container (name of the container/image or Image/Container instance from conu)
                    or path or file-like object for dockerfile
    :param group: str (name of the folder with group of checks, if None, all of them will be checked.)
    :param severity: str (if not None, only those checks will be run -- optional x required x warn ...)
    :param tags: list of str (if not None, the checks will be filtered by tags.)
    :param ruleset_name: str (e.g. fedora; if None, default would be used)
    :param ruleset_file: fileobj
    :param logging_level: logging level (default logging.WARNING)
    :return: Results instance
    """
    _set_logging(level=logging_level)
    logger.debug("Checking started.")
    target = Target(target=target,
                    logging_level=logging_level)
    checks_to_run = _get_checks(target_type=target.target_type,
                                group=group,
                                severity=severity,
                                tags=tags,
                                ruleset_name=ruleset_name,
                                ruleset_file=ruleset_file)
    result = go_through_checks(target=target,
                               checks=checks_to_run)
    return result


def get_checks(target_type=None, group=None, severity=None, tags=None, ruleset_name=None, ruleset_file=None,
               logging_level=logging.WARNING):
    """
    Get the sanity checks for the target.

    :param target_type: TargetType enum
    :param group: str (name of the folder with group of checks, if None, all of them will be checked.)
    :param severity: str (if not None, only those checks will be run -- optional x required x warn ...)
    :param tags: list of str (if not None, the checks will be filtered by tags.)
    :param ruleset_name: str (e.g. fedora; if None, default would be used)
    :param ruleset_file: fileobj
    :param logging_level: logging level (default logging.WARNING)
    :return: list of groups of checks
    """
    _set_logging(level=logging_level)
    logger.debug("Finding checks started.")
    return _get_checks(target_type=target_type,
                       group=group,
                       severity=severity,
                       tags=tags,
                       ruleset_name=ruleset_name,
                       ruleset_file=ruleset_file)


def _get_checks(target_type, group=None, severity=None, tags=None, ruleset_name=None, ruleset_file=None):
    ruleset = Ruleset(ruleset_name=ruleset_name,
                      ruleset_file=ruleset_file)
    return ruleset.get_checks(group=group,
                              severity=severity,
                              tags=tags,
                              target_type=target_type)


def _set_logging(
        logger_name="colin",
        level=logging.INFO,
        handler_class=logging.StreamHandler,
        handler_kwargs=None,
        format='%(asctime)s.%(msecs).03d %(filename)-17s %(levelname)-6s %(message)s',
        date_format='%H:%M:%S'):
    """
    Set personal logger for this library.

    :param logger_name: str, name of the logger
    :param level: int, see logging.{DEBUG,INFO,ERROR,...}: level of logger and handler
    :param handler_class: logging.Handler instance, default is StreamHandler (/dev/stderr)
    :param handler_kwargs: dict, keyword arguments to handler's constructor
    :param format: str, formatting style
    :param date_format: str, date style in the logs
    """
    if level != logging.NOTSET:
        logger = logging.getLogger(logger_name)
        logger.setLevel(level)

        handler_kwargs = handler_kwargs or {}
        handler = handler_class(**handler_kwargs)
        handler.setLevel(level)

        formatter = logging.Formatter(format, date_format)
        handler.setFormatter(formatter)
        logger.addHandler(handler)



# -*- coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from ..check_utils import check_label
from ..result import CheckResult
from .containers import ContainerCheck
from .dockerfile import DockerfileCheck
from .images import ImageCheck


class LabelCheck(ContainerCheck, ImageCheck, DockerfileCheck):

    def __init__(self, name, message, description, reference_url, tags, label, required, value_regex=None):
        super(LabelCheck, self) \
            .__init__(name, message, description, reference_url, tags)
        self.label = label
        self.required = required
        self.value_regex = value_regex

    def check(self, target):
        passed = check_label(label=self.label,
                             required=self.required,
                             value_regex=self.value_regex,
                             labels=target.labels)

        return CheckResult(ok=passed,
                           severity=self.severity,
                           description=self.description,
                           message=self.message,
                           reference_url=self.reference_url,
                           check_name=self.name,
                           logs=[])


class DeprecatedLabelCheck(ContainerCheck, ImageCheck, DockerfileCheck):

    def __init__(self, name, message, description, reference_url, tags, old_label, new_label):
        super(DeprecatedLabelCheck, self) \
            .__init__(name, message, description, reference_url, tags)
        self.old_label = old_label
        self.new_label = new_label

    def check(self, target):
        labels = target.labels
        old_present = labels is not None and self.old_label in labels

        passed = (not old_present) or (self.new_label in labels)

        return CheckResult(ok=passed,
                           severity=self.severity,
                           description=self.description,
                           message=self.message,
                           reference_url=self.reference_url,
                           check_name=self.name,
                           logs=[])
